@@ -18,14 +18,14 @@
 //! This module defines
 //! 1) a list of constants for every keyword
 //! 2) an `ALL_KEYWORDS` array with every keyword in it
-//!     This is not a list of *reserved* keywords: some of these can be
-//!     parsed as identifiers if the parser decides so. This means that
-//!     new keywords can be added here without affecting the parse result.
+//!    This is not a list of *reserved* keywords: some of these can be
+//!    parsed as identifiers if the parser decides so. This means that
+//!    new keywords can be added here without affecting the parse result.
 //!
-//!     As a matter of fact, most of these keywords are not used at all
-//!     and could be removed.
+//!    As a matter of fact, most of these keywords are not used at all
+//!    and could be removed.
 //! 3) a `RESERVED_FOR_TABLE_ALIAS` array with keywords reserved in a
-//!     "table alias" context.
+//!    "table alias" context.
 
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
@@ -84,6 +84,7 @@ define_keywords!(
     AFTER,
     AGAINST,
     AGGREGATION,
+    ALERT,
     ALGORITHM,
     ALIAS,
     ALL,
@@ -96,6 +97,7 @@ define_keywords!(
     ANY,
     APPLICATION,
     APPLY,
+    APPLYBUDGET,
     ARCHIVE,
     ARE,
     ARRAY,
@@ -109,6 +111,8 @@ define_keywords!(
     AT,
     ATOMIC,
     ATTACH,
+    AUDIT,
+    AUTHENTICATION,
     AUTHORIZATION,
     AUTO,
     AUTOINCREMENT,
@@ -117,6 +121,7 @@ define_keywords!(
     AVRO,
     BACKWARD,
     BASE64,
+    BASE_LOCATION,
     BEFORE,
     BEGIN,
     BEGIN_FRAME,
@@ -127,14 +132,18 @@ define_keywords!(
     BIGINT,
     BIGNUMERIC,
     BINARY,
+    BIND,
     BINDING,
     BIT,
     BLOB,
     BLOCK,
+    BLOOM,
     BLOOMFILTER,
     BOOL,
     BOOLEAN,
     BOTH,
+    BOX,
+    BRIN,
     BROWSE,
     BTREE,
     BUCKET,
@@ -150,8 +159,10 @@ define_keywords!(
     CASCADE,
     CASCADED,
     CASE,
+    CASES,
     CAST,
     CATALOG,
+    CATALOG_SYNC,
     CATCH,
     CEIL,
     CEILING,
@@ -162,11 +173,13 @@ define_keywords!(
     CHANNEL,
     CHAR,
     CHARACTER,
+    CHARACTERISTICS,
     CHARACTERS,
     CHARACTER_LENGTH,
     CHARSET,
     CHAR_LENGTH,
     CHECK,
+    CIRCLE,
     CLEAR,
     CLOB,
     CLONE,
@@ -185,6 +198,7 @@ define_keywords!(
     COMMENT,
     COMMIT,
     COMMITTED,
+    COMPATIBLE,
     COMPRESSION,
     COMPUTE,
     CONCURRENTLY,
@@ -192,6 +206,8 @@ define_keywords!(
     CONFLICT,
     CONNECT,
     CONNECTION,
+    CONNECTOR,
+    CONNECT_BY_ROOT,
     CONSTRAINT,
     CONTAINS,
     CONTINUE,
@@ -237,6 +253,7 @@ define_keywords!(
     DAYOFWEEK,
     DAYOFYEAR,
     DAYS,
+    DCPROPERTIES,
     DEALLOCATE,
     DEC,
     DECADE,
@@ -281,6 +298,7 @@ define_keywords!(
     ELEMENT,
     ELEMENTS,
     ELSE,
+    ELSEIF,
     EMPTY,
     ENABLE,
     ENABLE_SCHEMA_EVOLUTION,
@@ -305,12 +323,15 @@ define_keywords!(
     ESTIMATE,
     EVENT,
     EVERY,
+    EVOLVE,
     EXCEPT,
     EXCEPTION,
+    EXCHANGE,
     EXCLUDE,
     EXCLUSIVE,
     EXEC,
     EXECUTE,
+    EXECUTION,
     EXISTS,
     EXP,
     EXPANSION,
@@ -320,8 +341,10 @@ define_keywords!(
     EXTENDED,
     EXTENSION,
     EXTERNAL,
+    EXTERNAL_VOLUME,
     EXTRACT,
     FAIL,
+    FAILOVER,
     FALSE,
     FETCH,
     FIELDS,
@@ -357,6 +380,7 @@ define_keywords!(
     FREEZE,
     FROM,
     FSCK,
+    FULFILLMENT,
     FULL,
     FULLTEXT,
     FUNCTION,
@@ -367,6 +391,8 @@ define_keywords!(
     GENERATED,
     GEOGRAPHY,
     GET,
+    GIN,
+    GIST,
     GLOBAL,
     GRANT,
     GRANTED,
@@ -386,6 +412,8 @@ define_keywords!(
     HOSTS,
     HOUR,
     HOURS,
+    HUGEINT,
+    ICEBERG,
     ID,
     IDENTITY,
     IDENTITY_INSERT,
@@ -394,6 +422,8 @@ define_keywords!(
     ILIKE,
     IMMEDIATE,
     IMMUTABLE,
+    IMPORT,
+    IMPORTED,
     IN,
     INCLUDE,
     INCLUDE_NULL_VALUES,
@@ -401,15 +431,18 @@ define_keywords!(
     INDEX,
     INDICATOR,
     INHERIT,
+    INHERITS,
     INITIALLY,
     INNER,
     INOUT,
     INPATH,
+    INPLACE,
     INPUT,
     INPUTFORMAT,
     INSENSITIVE,
     INSERT,
     INSTALL,
+    INSTANT,
     INSTEAD,
     INT,
     INT128,
@@ -421,6 +454,7 @@ define_keywords!(
     INT64,
     INT8,
     INTEGER,
+    INTEGRATION,
     INTERPOLATE,
     INTERSECT,
     INTERSECTION,
@@ -457,9 +491,11 @@ define_keywords!(
     LIKE,
     LIKE_REGEX,
     LIMIT,
+    LINE,
     LINES,
     LIST,
     LISTEN,
+    LISTING,
     LN,
     LOAD,
     LOCAL,
@@ -477,7 +513,10 @@ define_keywords!(
     LOWER,
     LOW_PRIORITY,
     LS,
+    LSEG,
     MACRO,
+    MANAGE,
+    MANAGED,
     MANAGEDLOCATION,
     MAP,
     MASKING,
@@ -497,8 +536,10 @@ define_keywords!(
     MEDIUMTEXT,
     MEMBER,
     MERGE,
+    MESSAGE,
     METADATA,
     METHOD,
+    METRIC,
     MICROSECOND,
     MICROSECONDS,
     MILLENIUM,
@@ -515,12 +556,14 @@ define_keywords!(
     MODIFIES,
     MODIFY,
     MODULE,
+    MONITOR,
     MONTH,
     MONTHS,
     MSCK,
     MULTISET,
     MUTATION,
     NAME,
+    NAMES,
     NANOSECOND,
     NANOSECONDS,
     NATIONAL,
@@ -528,6 +571,7 @@ define_keywords!(
     NCHAR,
     NCLOB,
     NESTED,
+    NETWORK,
     NEW,
     NEXT,
     NFC,
@@ -561,6 +605,7 @@ define_keywords!(
     NUMERIC,
     NVARCHAR,
     OBJECT,
+    OBJECTS,
     OCCURRENCES_REGEX,
     OCTETS,
     OCTET_LENGTH,
@@ -575,8 +620,11 @@ define_keywords!(
     ONLY,
     OPEN,
     OPENJSON,
+    OPERATE,
     OPERATOR,
+    OPTIMIZATION,
     OPTIMIZE,
+    OPTIMIZED,
     OPTIMIZER_COSTS,
     OPTION,
     OPTIONS,
@@ -584,16 +632,22 @@ define_keywords!(
     ORC,
     ORDER,
     ORDINALITY,
+    ORGANIZATION,
     OUT,
     OUTER,
+    OUTPUT,
     OUTPUTFORMAT,
     OVER,
     OVERFLOW,
     OVERLAPS,
     OVERLAY,
+    OVERRIDE,
     OVERWRITE,
     OWNED,
     OWNER,
+    OWNERSHIP,
+    PACKAGE,
+    PACKAGES,
     PARALLEL,
     PARAMETER,
     PARQUET,
@@ -601,6 +655,7 @@ define_keywords!(
     PARTITION,
     PARTITIONED,
     PARTITIONS,
+    PASSING,
     PASSWORD,
     PAST,
     PATH,
@@ -617,7 +672,10 @@ define_keywords!(
     PLACING,
     PLAN,
     PLANS,
+    POINT,
     POLICY,
+    POLYGON,
+    POOL,
     PORTION,
     POSITION,
     POSITION_REGEX,
@@ -630,6 +688,7 @@ define_keywords!(
     PRESERVE,
     PREWHERE,
     PRIMARY,
+    PRINT,
     PRIOR,
     PRIVILEGES,
     PROCEDURE,
@@ -637,11 +696,13 @@ define_keywords!(
     PROGRAM,
     PROJECTION,
     PUBLIC,
+    PURCHASE,
     PURGE,
     QUALIFY,
     QUARTER,
     QUERY,
     QUOTE,
+    RAISE,
     RAISERROR,
     RANGE,
     RANK,
@@ -670,6 +731,7 @@ define_keywords!(
     RELATIVE,
     RELAY,
     RELEASE,
+    RELEASES,
     REMOTE,
     REMOVE,
     RENAME,
@@ -678,12 +740,16 @@ define_keywords!(
     REPEATABLE,
     REPLACE,
     REPLICA,
+    REPLICATE,
     REPLICATION,
     RESET,
+    RESOLVE,
+    RESOURCE,
     RESPECT,
     RESTART,
     RESTRICT,
     RESTRICTED,
+    RESTRICTIONS,
     RESTRICTIVE,
     RESULT,
     RESULTSET,
@@ -732,6 +798,7 @@ define_keywords!(
     SERDE,
     SERDEPROPERTIES,
     SERIALIZABLE,
+    SERVICE,
     SESSION,
     SESSION_USER,
     SET,
@@ -739,7 +806,10 @@ define_keywords!(
     SETS,
     SETTINGS,
     SHARE,
+    SHARED,
+    SHARING,
     SHOW,
+    SIGNED,
     SIMILAR,
     SKIP,
     SLOW,
@@ -752,6 +822,7 @@ define_keywords!(
     SPATIAL,
     SPECIFIC,
     SPECIFICTYPE,
+    SPGIST,
     SQL,
     SQLEXCEPTION,
     SQLSTATE,
@@ -771,17 +842,21 @@ define_keywords!(
     STDOUT,
     STEP,
     STORAGE_INTEGRATION,
+    STORAGE_SERIALIZATION_POLICY,
     STORED,
+    STRAIGHT_JOIN,
     STRICT,
     STRING,
     STRUCT,
     SUBMULTISET,
+    SUBSTR,
     SUBSTRING,
     SUBSTRING_REGEX,
     SUCCEEDS,
     SUM,
     SUPER,
     SUPERUSER,
+    SUPPORT,
     SUSPEND,
     SWAP,
     SYMMETRIC,
@@ -794,6 +869,7 @@ define_keywords!(
     TABLESAMPLE,
     TAG,
     TARGET,
+    TASK,
     TBLPROPERTIES,
     TEMP,
     TEMPORARY,
@@ -807,6 +883,7 @@ define_keywords!(
     TIME,
     TIMESTAMP,
     TIMESTAMPTZ,
+    TIMESTAMP_NTZ,
     TIMETZ,
     TIMEZONE,
     TIMEZONE_ABBR,
@@ -819,6 +896,7 @@ define_keywords!(
     TO,
     TOP,
     TOTALS,
+    TRACE,
     TRAILING,
     TRANSACTION,
     TRANSIENT,
@@ -836,7 +914,9 @@ define_keywords!(
     TRY_CONVERT,
     TUPLE,
     TYPE,
+    UBIGINT,
     UESCAPE,
+    UHUGEINT,
     UINT128,
     UINT16,
     UINT256,
@@ -859,6 +939,7 @@ define_keywords!(
     UNNEST,
     UNPIVOT,
     UNSAFE,
+    UNSET,
     UNSIGNED,
     UNTIL,
     UPDATE,
@@ -869,6 +950,8 @@ define_keywords!(
     USER,
     USER_RESOURCES,
     USING,
+    USMALLINT,
+    UTINYINT,
     UUID,
     VACUUM,
     VALID,
@@ -877,6 +960,7 @@ define_keywords!(
     VALUES,
     VALUE_OF,
     VARBINARY,
+    VARBIT,
     VARCHAR,
     VARIABLES,
     VARYING,
@@ -885,11 +969,14 @@ define_keywords!(
     VERBOSE,
     VERSION,
     VERSIONING,
+    VERSIONS,
     VIEW,
     VIEWS,
     VIRTUAL,
     VOLATILE,
+    VOLUME,
     WAREHOUSE,
+    WAREHOUSES,
     WEEK,
     WEEKS,
     WHEN,
@@ -904,6 +991,8 @@ define_keywords!(
     WORK,
     WRITE,
     XML,
+    XMLNAMESPACES,
+    XMLTABLE,
     XOR,
     YEAR,
     YEARS,
@@ -952,6 +1041,8 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     Keyword::ANTI,
     Keyword::SEMI,
     Keyword::RETURNING,
+    Keyword::ASOF,
+    Keyword::MATCH_CONDITION,
     // for MSSQL-specific OUTER APPLY (seems reserved in most dialects)
     Keyword::OUTER,
     Keyword::SET,
@@ -1008,7 +1099,7 @@ pub const RESERVED_FOR_COLUMN_ALIAS: &[Keyword] = &[
     Keyword::END,
 ];
 
-// Global list of reserved keywords alloweed after FROM.
+// Global list of reserved keywords allowed after FROM.
 // Parser should call Dialect::get_reserved_keyword_after_from
 // to allow for each dialect to customize the list.
 pub const RESERVED_FOR_TABLE_FACTOR: &[Keyword] = &[
